# Usage

The server.jar can be used to simulate games and tournaments or to visualize a game or a replay. To run it, you need Java (Version 7 or higher). If you want to start the program with the menu, you can just double click on the server.jar file. If you want to use arguments (for example to watch a replay), you have to start it in a console (Windows: Cmd, Mac: Terminal, Linux: Shell) using the command `java -jar server.jar arguments` where you replace `arguments` with the arguments you want to use.

# Examples / QuickStart
This examples demonstrate how to use different features of the program. This list is not complete but helps to get started.

* `java -jar server.jar -log logConfig.txt -commands commandsLocal.txt -map customMaps/circle.png -size 50 30 -players 2`: Start the visualization using the provided config files (`logConfig.txt` and `commandsLocal.txt`), the map `circle.png` with a size of 50x30 fields for two players.
* `java -jar server.jar -replay log/replay.txt`: replay the game saved in the file `log/replay.txt`. This is the default location for a replay file defined in the provided `logConfig.txt`. You have to play a game to create a replay before using this command.
* `java -jar server.jar -log logConfig.txt -commands commands.txt -map customMaps/circle.png -size 50 30 -players 2 -timelimit 100`: Almost the same as the first one. Use a different set of players (defined in `commands.txt`. Add your bot to this file to test it. If you want to use the already entered players, you have to compile the sample bots) and give the players more time to "think" (Use this option if your program is slow but you want to test it anyways. Default value is 10 seconds which should be enough for most programs).

# Arguments
Server.jar supports the following arguments:

* `-replay replayFile`: replays the given replayFile. The replayFile will be written to the `Replay` log channel.
* `-log logConfigFile`: loads the given config file for logs.
* `-map mapFile`: the image file to use as a map.
* `-players count`: the number of players participating in one game. If more players are given, the first few players will be chosen.
* `-commands commandsFile`: file containing the commands used to run the players.
* `-size width height`: the size in fields of the map. The image will be resized accordingly.
* `-lap count`: the number of laps to simulate.
* `-timelimit seconds`: the time each external player (i.e. an AI) is allowed to use during the whole game ("chess clock").
* `-extsleep milliseconds`: the time to wait after a move of an external player (i.e. an AI; this simulates that it has to "think" about it's move instead of playing instantly).
* `-tournament tournamentFile`: file specifying the tournament.
* `-fastgui`: draw a slimmer gui that is faster.
* `-nogui`: do not show a gui.
* `-nametags`: show nametags above each player.

## Controls
Use the mouse to choose a field for an action (e.g. move). Press M (Move), B (Bomb) or F (Freezer) to change the desired action, M is the default action. After an M action, the next player's turn will start. You can use `PageUp` and `PageDown` to zoom and the arrow keys to move around.

While in replay mode, there is a second window where you can jump to a frame (top slider) and adjust the speed (bottom slider).

## Map format
The maps can be either in png format (see below) or using the format used in the protocol. Such a file starts with two integers on the first line, the width and the height. On the following lines the contain the map in the format specified in the protocol. Preferably use the image format, as it can be scales as needed.

## Image format
The maps are saved in a png file. The image will be scaled as needed(Make sure to specify -size). The different blocks are represented by different colors:

* `#FFFFFF` empty cell
* `#000000` blocked cell
* `#00FF00` start cell
* `#FF0000` dest cell
* `#abab??` checkpoint (replace `ab` with value between `01` and `1A` and replace `??` with an arbitrary value)

## Log Config
To configure the logger one can specify a config file. This file follows the following format:

The first line contains the folder to put the logs into. The next lines contain each a definition of a log channel, using `O E F` where `O` is `0/1` (printing to stdout), `E` is `0/1` (printing to stderr and `F` is a file(suffix) (the name of the file to write the log to). If no file should be written, use `:` as file name.

The available log channels are (in this order):

* `DEBUG`: Debug output of the server
* `GAME`: Log of a game (contains the same information as `LEVEL_REPLAY` but is easier to read by humans)
* `BEHAVIOUR` Behaviour of AIs (e.g. if the crash)
* `POCESSERR` Std Err of the AIs
* `EVALUATION` Log of an evaluation
* `TOURNAMENT` Log of a tournament
* `EXTERNAL_DEBUG` The whole communication between the server and AIs (This gets really big and is mostly not used)
* `LEVEL_REPLAY` Log used to replay a game (contains the same information as `GAME` but is easier to parse)

If you want to submit a played game, you have to upload the file specified for `LEVEL_REPLAY` (default in `logConfig.txt` is `replay.txt`).

## Commands File
The commands file contains all commands used to run the players. The file specifies alternating the name of a player and the command to start, starting with a player name. The command is either a path to an executable (you can also specify arguments, so `python awesomeBot.py` works as well) or `:local` for a player that plays using the visualization. Note that the simulation will block when using a `:local` player without a gui.

## Tournament File
The tournament file contains all settings to test the given players on. Each line specifies one setting using `M W H P` where `M` is the name of the map image, `W` and `H` are the size of the map and `P` is the number of players per game. In a tournament all possible combinations of players with the given player count are played.